Include "../monopole_data.pro";  

fac = s;
flag = 1; // if 0, extruded mesh becomes free in the monopole

lcdAntennaZero = a/4;
lcdAntennaA = lcdAntennaZero;

If (_use_fineMesh == 1)
	lambdaMin = c0/fmax;
	lcd = fac*lambdaMin/nopPerLambdaAir;
	lcair = lcd;
	nopL = flag*(Ceil[L/lcdAntennaZero]);  // to make the antenna very fine along L
Else
	lcdAntennaZero = 4*lcdAntennaZero;
	lambdaMin = c0/fmax;
	lcair = fac*lambdaMin/nopPerLambdaAir;

	delta = Sqrt(2.0*nuMonopole/(2*Pi*Freq*sigmaMonopole));
	lcdAntennaA = fac*delta/nbDelta;

	lcair1 = lambdaMin/10;
	nopL = fac*6*flag*(Ceil[L/lcair1]);
EndIf
nopL = (nopL < 10) ? 10 : nopL;

Printf("nopL %g",nopL);
Printf("lcdAntennaA %g lcdAntennaZero %g ratio %g", lcdAntennaA, lcdAntennaZero, lcdAntennaZero/lcdAntennaA);

//=================================================
// Monopole
//=================================================

d = 0;  // no gap
yd = (d >= 0)? d:0.;
p0 = newp ; Point(p0) = {0, yd, 0, lcdAntennaZero};
p1 = newp ; Point(p1) = {a, yd, 0, lcdAntennaA};
LineMonopoleBottom = newl ; Line(LineMonopoleBottom) = {p0,p1};

surf[] = Extrude {0, L, 0} {
  Line{LineMonopoleBottom}; Layers{nopL} ;
};
surfMonopole[0] = surf[1] ; LineMonopoleTop = surf[0] ;

p_[] = Boundary{Line{LineMonopoleTop};};

interfaceMonopoleAir[] = Boundary{ Surface{surfMonopole[]};} ;

axisMonopole[] = interfaceMonopoleAir[{3}] ;
feedMonople[] = interfaceMonopoleAir[{0}] ;
interfaceMonopoleAir[] -= axisMonopole[];
interfaceMonopoleAir[] -= feedMonople[];

//=================================================
// Air 
//=================================================

pZero = newp; Point(pZero) = { 0, 0, 0, lcdAntennaZero};

pp[]+=p1;
pp[]+=newp; Point(newp) = {  b, 0, 0, lcdAntennaZero};

// reduce air around
cc = 1;
pp[]+=newp; Point(newp) = { xb/cc, 0, 0, lcair};
pp[]+=newp; Point(newp) = { xb/cc, L, 0, lcair};
pp[]+=newp; Point(newp) = { 0, xb/cc+L, 0, lcair};
pp[]+=p_[0];
	
For k In {0:2}
	lbox[]+=newl; Line(newl) = {pp[k],pp[k+1]};
EndFor
lbox[]+=newl; Circle(newl) = {pp[3],p_[0],pp[4]};
lbox[]+=newl; Line(newl) = {pp[4],pp[5]};
interfaceAirPML[] = lbox[{2,3}];  // it will be the inf boundary if no PML is used

surfAirLL = newll ; Line Loop(surfAirLL) = {lbox[],-interfaceMonopoleAir[]};
surfAir = news ; Plane Surface(surfAir) = {surfAirLL};


// For aestetics
Mesh.Light = 0;  // so that to see mesh colors in the coax part

Recursive Color SkyBlue { Surface{surfAir};}
Recursive Color Black  { Surface{surfMonopole};}


//=================================================
// Physical regions
//=================================================

// surfaces
Physical Surface(MONOPOLE) = {surfMonopole[]};        
Physical Surface(AIR) = surfAir;                       

// boundaries
Physical Line(TERMINAL) = {feedMonople[]}; 
Physical Line(CUT) = {lbox[{0}]};

Physical Line(AXIS) = {lbox[{4}],axisMonopole[]};

If (Flag_AnalysisTypeExtBnd == 1) 
	Physical Line(GROUND) = {lbox[{1}],lbox[{2}],lbox[{3}]};
Else
	Physical Line(SURFAIRINF) = interfaceAirPML[];
    Physical Line(GROUND) = {lbox[{1}]};	
EndIf
